/**
 * common code that can be imported from server code and client code
 */
import { NeededInfoFor, ToCompiletime } from "../../TypedServer";
import { ChatMessage } from "../basic-chat/common";
import { ServerSideClientSocket, RootServer, ServerDefinition, SimpleNamespace } from "../..";
import * as t from "io-ts";
export { ChatMessage };
export declare const runtimeSchema: {
    ServerMessages: {
        chatMessage: t.Type<ChatMessage, ChatMessage, unknown>;
        history: t.Type<ChatMessage[], ChatMessage[], unknown>;
    };
    ClientRPCs: {
        postMessage: {
            request: t.ExactC<t.TypeC<{
                message: t.StringC;
                channel: t.UnionC<[t.LiteralC<"en">, t.LiteralC<"ru">]>;
            }>>;
            response: t.Type<"ok", "ok", unknown>;
        };
    };
    ClientMessages: {};
};
export declare type ChatNamespace = SimpleNamespace<ToCompiletime<typeof runtimeSchema>>;
export interface MyServerDefinition extends ServerDefinition {
    namespaces: {
        "/chat": ChatNamespace;
    };
}
export declare type ChatServerInfo = NeededInfoFor<MyServerDefinition, "/chat">;
export declare type ChatSocket = ServerSideClientSocket<MyServerDefinition, "/chat">;
export declare type RPCs = ChatServerInfo["NamespaceSchema"]["ClientRPCs"];
export declare type Req<K extends keyof RPCs> = RPCs[K]["request"];
export declare type Res<K extends keyof RPCs> = RPCs[K]["response"];
export declare type MyRootServer = RootServer<ServerDefinition>;
